/* Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.Array;
import java.lang.reflect.Type;
import java.math.BigDecimal;
import java.security.Principal;
import java.sql.Date;
import java.sql.Time;
import java.sql.Timestamp;
import java.sql.DriverManager;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletInputStream;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpSession;

import jp.littlesoft.data.Column;
import jp.littlesoft.data.ITableDef;
import jp.littlesoft.data.Row;
import jp.littlesoft.data.Table;
import jp.littlesoft.data.ITableDef.TableDef;
import jp.littlesoft.data.exception.DataException;
import jp.littlesoft.sql.SQL;
import jp.littlesoft.web.WebDataTable;
import jp.littlesoft.web.WebSQLTable;

public class TableTest {

  public static class Base implements ITableDef {
    private static final long serialVersionUID = 6062483674222162851L;

    @ColumnDef(caption="登録日時" , formatPattern="GGyy/MM/dd hh:mm:ss.SSS")
    public java.sql.Timestamp createDatetime = new java.sql.Timestamp(System.currentTimeMillis());

    @ColumnDef(caption="更新日時" , formatPattern="GGyy/MM/dd hh:mm:ss.SSS")
    public java.sql.Timestamp updateDatetime = new java.sql.Timestamp(System.currentTimeMillis());

    @SuppressWarnings("unchecked")
    @Override
    public MetaData<?> createMetaData() {
      return new MetaData(this.getClass());
    }
  }

  @TableDef(schema="PUBLIC", name="T_EMP")
  public static class Employee extends Base {

    private static final long serialVersionUID = 3916988778547072145L;

    @ColumnDef(caption="社員番号")
    public Long empId;

    @ColumnDef(caption="社員名")
    public String name;

    @ColumnDef(caption="所属コード")
    public Integer deptCode;

    @ColumnDef(caption="入社日" , formatPattern="GGyy/MM/dd")
    public java.sql.Date enterDate;

    @ColumnDef(caption="性別")
    public Short sex;
    @ColumnDef(caption="給与")
    public BigDecimal salary = new BigDecimal("0");
    @ColumnDef(caption="備考")
    public String memo;

    @ColumnDef(caption="賞与" , formatPattern="#,##0")
    public BigDecimal getBonus() {
      try {
        return new BigDecimal(new Integer((int)(salary.longValue() * 13.245)));
      } catch (Exception x) {
        return null;
      }
    }

    @SuppressWarnings("serial")
    public MetaData<?> createMetaData() {
      return new MetaData<Employee>() {{
        addPrimaryKey($$.empId);
        addForeginKey($$.deptCode)
          .references(Department.class).onDelete(FKOption.RESTRICT)
          .usingIndex(true);
        addIndex($$.name)
          .unique(false);
      }};
    }
  }

  public static class EmployeeBackup extends Employee {
    private static final long serialVersionUID = -3271748801137301335L;
    public MetaData<?> createMetaData() {
      return new MetaData<EmployeeBackup>();
    }
  }

  @TableDef(schema="PUBLIC", name="TEST")
  public static class User extends Base{
    private static final long serialVersionUID = 4380358529045912816L;
    public Integer id;
    public String name;
    public MetaData<?> createMetaData() {
      return new MetaData<User>();
    }
  }


  public static class Department  extends Base {
    private static final long serialVersionUID = 6582271562474380334L;
    public Integer code;
    public String name;
    public Integer parentCode;
    public MetaData<?> createMetaData() {
      return new MetaData<Department>();
    }
  }

  public static class EmployeeView extends Employee {
    private static final long serialVersionUID = -4366935193649100148L;

    @ColumnDef(caption="所属名")
    public String deptName;

    @ColumnDef(caption="上位所属名")
    public String parentDeptName;
    public MetaData<?> createMetaData() {
      return new MetaData<EmployeeView>();
    }
  }

  @SuppressWarnings("serial")
  static void testEvaluateNonNulls(java.sql.Connection conn, Integer deptCode) throws Exception {
    WebSQLTable<Employee> sql = new WebSQLTable<Employee>() {{
      select()
        .whereEvaluateNonNulls($$.deptCode).is(":deptCode")
      .end.orderBy($$.empId)
      .prepare();
    }};
    sql.getParams().setValue("deptCode", deptCode);
    sql.activate(conn);
    sql.dump();
  }
  
  @SuppressWarnings("serial")
  static void testDynamicCondtion(java.sql.Connection conn, Integer deptCode) throws Exception {
    WebSQLTable<Employee> sql = new WebSQLTable<Employee>() {{
      select().whereDynamicCondition("key")
      .end.orderBy($$.empId)
      .prepare();
    }};
    sql.setDynamicCondtion("key", sql.$if(sql.$$.deptCode).is(deptCode));
    sql.activate(conn);
    sql.dump();
  } 
  
  @SuppressWarnings("serial")
  static void testRemoteJoin(java.sql.Connection conn, java.sql.Connection remoteConn) throws Exception {
    
  }
  
  /**
   * @param args
   */
  @SuppressWarnings("serial")
  public static void main(String[] args) {
    try {
      Class.forName("org.h2.Driver");
      java.sql.Connection conn = DriverManager.getConnection("jdbc:h2:file:C:/gomi/mydb", "sa", "");
      java.sql.Connection connRemote = DriverManager.getConnection("jdbc:h2:file:C:/gomi/remote", "sa", "");
      
      if (true) return;
      System.out.println("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@");
      
      System.out.println("+++ testEvaluateNonNulls(parameter is null) +++");
      testEvaluateNonNulls(conn, null);
      System.out.println("--- testEvaluateNonNulls(parameter is null)  ---");
      
      System.out.println("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@");
      
      System.out.println("+++ testEvaluateNonNulls(parameter is Integer(1)) +++");
      testEvaluateNonNulls(conn, new Integer(3));
      System.out.println("--- testEvaluateNonNulls(parameter is Integer(1))  ---");
      
      System.out.println("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@");
      
      System.out.println("+++ testDynamicCondtion(parameter is Integer(1)) +++");
      testDynamicCondtion(conn, new Integer(3));
      System.out.println("--- testDynamicCondtion(parameter is Integer(1))  ---");
      
      System.out.println("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@");

      System.out.println("@@@ UPDATE start");
      SQL<Employee> update1 = new SQL<Employee>(){{
        update().set(
            $($$.memo).accept( $($$.memo).add("2009年度昇給") ),
            $($$.salary).accept( $($$.salary).divide(1.25) )
        )
        .where($$.deptCode).is(2)
        .end
        .prepare();
      }};
      System.out.println("@@@ UPDATE 処理件数：" + update1.execute(conn, true, false));

      System.out.println("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@");
      
      if(true) return;
      
      
      

      
      
      SQL<Employee> select1 = new SQL<Employee>() {{
        select()
          .where($$.deptCode).in(1,2).end
          .orderBy($$.empId)
          .prepare();
      }};
      List<Employee> empList = select1.execute(conn);
      for (Employee e : empList) {
        System.out.println("id:" + e.empId + ", name:" + e.name +
            ", salary:" + e.salary + ", memo:" + e.memo + ", enterDate:" + e.enterDate + ", updateDatetime:" + e.updateDatetime);
      }
      System.out.println("@@@ 単純SELECT complete");

//      GsonBuilder builder = new GsonBuilder();
//      builder.registerTypeAdapter(Timestamp.class, new TimestampSerializer());
//      builder.registerTypeAdapter(Timestamp.class, new TimestampDeserializer());
//      builder.registerTypeAdapter(Date.class, new DateSerializer());
//      builder.registerTypeAdapter(Date.class, new DateDeserializer());
//      builder.registerTypeAdapter(Time.class, new TimeSerializer());
//      builder.registerTypeAdapter(Time.class, new TimeDeserializer());
//
//      Gson gson = builder.create();
//      String json = gson.toJson(empList.toArray(), new TypedArray(Employee.class).getType());
//      System.out.println(json);
      System.out.println("#######################################################");
      
      
      //gson.registerInstanceCreator(MyType.class, new MyInstanceCreator());
      
//      java.lang.reflect.Type collectionType = new TypeToken<Collection<Employee>>(){}.getType();
//      Collection<Employee> list2 = gson.fromJson(json, collectionType);
      //Employee[] list2 = (Employee[]) gson.fromJson(json, new TypedArray(Employee.class).getType());
      //for (Employee e : list2) {
      //  System.out.println("id:" + e.empId + ", name:" + e.name +
      //      ", salary:" + e.salary + ", memo:" + e.memo + ", enterDate:" + e.enterDate + ", updateDatetime:" + e.updateDatetime);
      //}
      
      //if (true) return;
      
//      System.out.println("@@@ INSERT SELECT start");
//      SQL<EmployeeBackup> insert1 = new SQL<EmployeeBackup>() {{
//        Employee employee = $useDummy(Employee.class);
//        insert().select().from(employee).prepare();
//      }};
//      System.out.println("@@@ INSERT SELECT 処理件数：" + insert1.execute(conn, true, false));

//      System.out.println("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@");
//      System.out.println("@@@ INSERT INTO start");
//      SQL<User> insert2 = new SQL<User>() {{
//        Employee employee = $useDummy(Employee.class);
//        insert().select(
//          $(employee.empId).into($$.id),
//          $(employee.name).into($$.name)
//        ).from(employee).prepare();
//      }};
//      System.out.println("@@@ INSERT INTO 処理件数：" + insert2.execute(conn, true, false));

      System.out.println("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@");
      System.out.println("@@@ INSERT VALUES start");
      SQL<Employee> insert3 = new SQL<Employee>() {{
        insert().values(
          $($$.empId).accept(800),
          $($$.name).accept("鳩山 邦彦"),
          $($$.sex).accept(1),
          $($$.deptCode).accept(1),
          $($$.salary).accept(new BigDecimal("250000")),
          $($$.enterDate).accept(new java.sql.Date(System.currentTimeMillis())),
          $($$.memo).accept("はろ' , 'はろ"),
          $($$.createDatetime).accept(new java.sql.Timestamp(System.currentTimeMillis())),
          $($$.updateDatetime).accept(new java.sql.Timestamp(System.currentTimeMillis()))
        ).prepare();
      }};
      System.out.println("@@@ INSERT VALUES 処理件数：" + insert3.execute(conn, true, false));

      System.out.println("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@");
      System.out.println("@@@ INSERT VALUES start");
      SQL<Employee> insert4 = new SQL<Employee>() {{
        insert().prepare();
      }};
      insert4.params.empId = new Long(900);
      insert4.params.name = "鳩山 邦彦 ";
      insert4.params.sex = 1;
      insert4.params.deptCode = 1;
      insert4.params.salary = new BigDecimal("250000");
      insert4.params.enterDate = new java.sql.Date(System.currentTimeMillis());
      insert4.params.memo = "はろはろ";
      insert4.params.createDatetime = new java.sql.Timestamp(System.currentTimeMillis());
      insert4.params.updateDatetime = new java.sql.Timestamp(System.currentTimeMillis());
      insert4.execute(conn, true, false);
      System.out.println("@@@ INSERT VALUES 処理件数：" + insert4.execute(conn, true, false));


      System.out.println("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@");
      System.out.println("@@@ UPDATE start");
      SQL<Employee> update0 = new SQL<Employee>(){{
        update().set(
            $($$.memo).accept( $($$.memo).add("2009年度昇給") ),
            $($$.salary).accept( $($$.salary).divide(1.25) )
        )
        .where($$.deptCode).is(2)
        .end
        .prepare();
      }};
      System.out.println("@@@ UPDATE 処理件数：" + update0.execute(conn, true, false));

      System.out.println("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@");
      System.out.println("@@@ DELETE start");
      SQL<Employee> delete1 = new SQL<Employee>() {{
        delete().where($$.empId).in(800,900).end
        .prepare();
      }};
      System.out.println("@@@ DELETE 処理件数：" + delete1.execute(conn, true, false));


      System.out.println("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@");
      System.out.println("@@@ 単純SELECT start");


      

      WebSQLTable<EmployeeView> tEmployee = new WebSQLTable<EmployeeView>(){{
        Employee    employee   = $useDummy(Employee.class);
        Department  department = $useDummy(Department.class);
        Department  parentDept = $useDummy(Department.class);

        select(
            $(employee).into($$),
            $(department.name).into($$.deptName),
            $(parentDept.name).into($$.parentDeptName)
          )
          .from(employee).leftOuterJoin(
            $(department).leftOuterJoin(parentDept).
              on(department.parentCode).is(parentDept.code).end
          )
          .on(employee.deptCode).is(department.code).end.
          where(
            $if(employee.deptCode).is(":dept1").
             or(employee.deptCode).is(":dept2")
          )
          .and(employee.salary).between(":sal1",":sal2")
          .end
          .orderBy(
            $(employee.empId).ascending(),
            $(employee.name).ascending().nullsLast()
          )
          .prepare();
          getColumn("parentDeptName").setNullable(true);
      }};
      tEmployee.getParams().setValue("dept1", new Long(1));
      tEmployee.getParams().setValue("dept2", new Long(2));
      tEmployee.getParams().setValue("sal1", new BigDecimal("0"));
      tEmployee.getParams().setValue("sal2", new BigDecimal("500000"));
      tEmployee.activate(conn);

      //List<Employee> empList = sqlSelect.execute(conn);
      for (Employee e : empList) {
        System.out.println("@@@@@@@@@@@@@ id:" + e.empId + ", name:" + e.name +
            ", salary:" + e.salary + ", memo:" + e.memo );
      }

//        .whereEvaluateNonNulls(sql.$$.enterDate).between(":from", ":to")
//        .whereEvaluateNonNulls(sql.$$.salary).bigger(":salary")
//          .andEvaluateNonNulls(sql.$$.deptCode).is(":deptCode")
////           .orEvaluateNonNulls(sql.$$.empId).in(":e1", ":e2", ":e3")
//       .end      
      
//       );
//
////      .whereEvaluateNonNulls(sql.$$.deptCode).is(":deptCode")
////          .and( sql.$if(sql.$$.empId).is(10)
////                .andDynamicCondition("dynCond1")  )
////          //.andDynamicCondition("dynCond1")
//
//


      //sql.params.salary = new BigDecimal("250000");
      //sql.getParams().setValue("e1", new Long(10));
      //sql.getParams().setValue("e2", new Long(20));

      for (Row<EmployeeView> row : tEmployee.visibleRows()) {
        row.startEditing();
        row.entity.salary = row.entity.salary.add(new BigDecimal(10));
        row.entity.updateDatetime = new java.sql.Timestamp(System.currentTimeMillis());
        row.entity.memo = row.entity.memo + "へっ？";
        if (row.entity.parentDeptName == null) row.entity.parentDeptName = "へっ？";
        row.validate();
      }

      EmployeeView newEmp = tEmployee.appendRow().entity;
      newEmp.empId = 26L;
      newEmp.name = "長尾★ 寿宏";
      newEmp.deptCode = 1;
      newEmp.enterDate = new java.sql.Date(System.currentTimeMillis());
      newEmp.sex = 1;
      newEmp.salary = new BigDecimal("1234567.12");
      newEmp.memo = "新入社員です";
      newEmp.createDatetime = new java.sql.Timestamp(System.currentTimeMillis());
      newEmp.updateDatetime = new java.sql.Timestamp(System.currentTimeMillis());
      newEmp.deptName = "?";
      newEmp.parentDeptName = "?";
      tEmployee.validate(newEmp);

      tEmployee.resolveChanges();

      System.out.println("[after]---------------------------------------------------------");
      tEmployee.activate();
      for (Row<EmployeeView> row : tEmployee.visibleRows()) {
        System.out.print("行番号[" +  (tEmployee.indexOf(row)+1) + "] {");
        for (int i=0; i<tEmployee.getColumnCount(); i++) {
          if (i > 0) System.out.print(", ");
          System.out.print(tEmployee.getColumn(i).getCaption() + " : " + row.getValueAsString(i));
        }
        System.out.println("}");
      }
      System.out.println("[emptyActivate]---------------------------------------------------------");
      tEmployee.emptyActivate(conn);
      for (Row<EmployeeView> row : tEmployee.visibleRows()) {

        System.out.print("[" + row.getRowID() + "] {");
        for (int i=0; i<tEmployee.getColumnCount(); i++) {
          if (i > 0) System.out.print(", ");
          System.out.print(tEmployee.getColumn(i).getCaption() + " : " + row.getValueAsString(i));
        }
        System.out.println("}");
      }
      System.out.println("end   @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@");
      conn.rollback();
      conn.close();
    } catch (Exception x) {
      x.printStackTrace();
    }
  }
}
