/* Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.data;

import java.io.Serializable;

import jp.littlesoft.web.HTML;


/**
 * コード定義クラス
 * 
 * @author LittleSoft Corporation
 * @version 1.3.0
 */
public class Code extends java.util.LinkedHashMap<String, String> implements Serializable {
  
  /**
   * コンストラクタ。
   */
  public Code() {
  }
  
  /**
   * 指定されたキーの値を取得する。
   * 
   * @return キーに対応する値
   */
  public String get(Object key) {
    if (key != null && key instanceof java.math.BigDecimal) {
      key = key.toString();
    }
    String value = super.get(key);
    if (value == null)
      value = "";
    return value;
  }
  
  /**
   * セレクトボックスのoptionタグを作成する。
   * 
   * @return optionタグ
   */
  public String getOption() {
    return this.getOption("");
  }
  
  /**
   * セレクトボックスのoptionタグを作成する。
   * 
   * @param defaultSelection 初期選択値
   * @return optionタグ
   */
  public String getOption(String defaultSelection) {
    String result = "";
    for (String key : this.keySet()) {
      result += "<option value=\"" + HTML.escape(key) + "\"";
      if (key != null && key.equals(defaultSelection)) {
        result += " selected ";
      }
      result += ">" + HTML.escape(this.get(key)) + "</option>";
    }
    return result;
  }
  
  /**
   * input(radio)タグを作成する。
   * 
   * @param name name属性
   * @param defaultSelection 初期選択値
   * @return input(radio)タグ
   */
  public String getRadio(String name, String defaultSelection) {
    return getRadio(name, defaultSelection, null);
  }
  
  /**
   * input(radio)タグを作成する。
   * 
   * @param name name属性
   * @param defaultSelection 初期選択値
   * @param classValue class属性
   * @return input(radio)タグ
   */
  public String getRadio(String name, String defaultSelection, String classValue) {
    String result = "";
    String classString  = "";
    if (classValue != null) {
      classString = " class=\"" + HTML.escape(classValue) + "\"";
    }
    for (String key : this.keySet()) {
      String id = name + "_" + key;
      result += "<input type=\"radio\" name=\"" + HTML.escape(name) + "\" value=\"" + HTML.escape(key)
          + "\" id=\"" + HTML.escape(id) + "\"" + classString;
      if (key != null && key.equals(defaultSelection)) {
        result += " checked ";
      }
      result += "><label for=\"" + HTML.escape(id) + "\">" + HTML.escape(this.get(key)) + "</label>";
    }
    return result;
  }
  
  /**
   * input(radio)タグを作成する。
   * 
   * @param name name属性
   * @param defaultSelection 初期選択値
   * @param classValue class属性
   * @param script onClickなどのイベント記述
   * @return input(radio)タグ
   */
  public String getRadio(String name, String defaultSelection, String classValue, String script) {
    String result = "";
    String classString  = "";
    if (classValue != null) {
      classString = " class=\"" + HTML.escape(classValue) + "\"";
    }
    for (String key : this.keySet()) {
      String id = name + "_" + key;
      result += "<input type=\"radio\" name=\"" + HTML.escape(name) + "\" value=\"" + HTML.escape(key)
          + "\" id=\"" + HTML.escape(id) + "\"" + classString;
      if (key != null && key.equals(defaultSelection)) {
        result += " checked ";
      }
      if (script != null) {
        result += script;
      }
      result += "><label for=\"" + HTML.escape(id) + "\">" + HTML.escape(this.get(key)) + "</label>";
    }
    return result;
  }
  
}
