/** Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.data.event;

import java.util.*;

/**
 * データ変更イベントリスナー。<br/><br/>
 *
 * Tableクラス、Paramsクラスなどのデータの内容が変更された際のイベントを受け取るためのインタフェースです。<br/>
 * このイベントを受け取るには、受け取り側クラスでChangeListenerインターフェースを実装し、
 * 対象のデータオブジェクトaddChangeListenerで受け取り側オブジェクトを登録します。<br/><br/>
 *
 * データ項目の値が変更された場合は、当クラスのoldValue､newValueプロパティに変更される前後のそれぞれの値が設定されます。<br/><br/>
 * 
 * @see jp.littlesoft.data.Table
 * @see jp.littlesoft.data.event.ChangeEvent
 * 
 * @author LittleSoft Corporation
 * @version 1.3.0
 */
public interface ChangeListener extends EventListener {

  /**
   * データオブジェクトの活性化が完了した際のイベント。<br/><br/>
   *
   * このイベントは表示コンポーネントの初期化に使用します。<br/>
   * 
   * @param  e  ChangeEvent
   * @exception  java.lang.Exception  例外
   */
  public void onActivate(ChangeEvent e) throws Exception;

  /**
   * データオブジェクトのカラム構造が決定した際のイベント。<br/><br/>
   *
   * このイベントは各カラムオブジェクトの属性設定で使用します。<br/>
   * 
   * @param  e  ChangeEvent
   */
  public void onColumnSetup(ChangeEvent e);

  /**
   * データオブジェクトが非活性化が完了したイベント。<br/>
   * 
   * @param e ChangeEvent
   */
  public void onPassivate(ChangeEvent e);

  /**
   * データオブジェクトのカラムに新しい値がセットされる直前のイベント。<br/><br/>
   *
   * このイベントはカラム値に対する特別なチェック処理等で利用できます。<br/>
   * ChangeEventのNewValueを検証し、必要であれば値の変更を行う事ができます。<br/>
   * また、このメソッド内でExceptionを送出するとカラムの変更を強制的にキャンセルする事ができます。<br/>
   * 
   * @param  e  ChangeEvent
   * @exception  java.lang.Exception  例外
   */
  public void beforeSetValue(ChangeEvent e) throws Exception;

  /**
   * データオブジェクトのカラムに新しい値がセットされた直後のイベント。<br/>
   *
   * @param  e  ChangeEvent
   */
  public void afterSetValue(ChangeEvent e);

}
