/** Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.data.event;

import java.util.*;

/**
 * テーブルオブジェクトデータ永続化のイベントリスナーインターフェース。<br/><br/>
 * 
 * このイベントを受け取るには、受け取り側クラスでResolvingListenerインターフェースを実装し、
 * 対象のデータオブジェクトaddResolvingListenerで受け取り側オブジェクトを登録します。<br/>
 * 
 * @author LittleSoft Corporation
 * @version 1.3.0
 */
public interface ResolvingListener extends EventListener {

  /**
   * 永続化処理前イベント。 <br/><br/>
   * 
   * 行単位で永続化直前のイベントです。<br/>
   *
   * @param  e  ResolvingEvent
   */
  public void rowResolving(ResolvingEvent e);

  /**
   * 永続化処理エラーイベント。<br/><br/>
   * 
   * 行単位で永続化の際、エラーとなった場合のイベントです。<br/>
   * 
   * @param  e  ResolvingEvent
   * @param  ex Exception
   */
  public void rowResolvingError(ResolvingEvent e, Exception ex);

  /**
   * 永続化処理後イベント。<br/>
   * 
   * @param  e  ResolvingEvent
   */
  public void rowResolved(ResolvingEvent e);
}
