/** Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.data.validator;

import jp.littlesoft.data.Row;
import jp.littlesoft.data.Column;
import jp.littlesoft.data.exception.InvalidValueException;

/**
 * カラム値の検証を行うクラスのインタフェース. <br/><br/>
 * validate()メソッドに検証ロジックを記述します。<br/>
 * ColumnオブジェクトにValidatorをセットすると、値設定時に自動的に当クラスのvalidateメソッドが呼び出されます。<br/>
 *
 * <pre>
 * // 独自Validatorクラスの実装 
 * public void MyValidator extends DefaultValidator {
 *
 *   public void validate(Column column, Object value) .. {
 *     // MyValidator独自の検証
 *     ......
 *     // デフォルトの検証ロジックを実行
 *     super.validate(column, value);
 *   }
 * }
 * -------
 * // ChangeListener実装クラスでtableのcolAにMyValidatorを設定する。
 * public void onColumnSetup(..) ... {
 *   table.getColumn("colA").setValidator( new MyValidator() );
 *     :
 * }
 * </pre>
 * 
 * @see jp.littlesoft.data.Column#setConstraint
 * @see jp.littlesoft.data.Column#setValidator
 * 
 * @author LittleSoft Corporation
 * @version 1.3.0
 */
public interface Validator extends java.io.Serializable {

  /**
   * 検証を行う。<br/>
   *
   * @param  column カラム
   * @param  value 検証対象値
   *
   * @throws jp.littlesoft.data.exception.InvalidValueException エラーが検出された場合に送出される。
   */
	@SuppressWarnings("unchecked")
  public void execute(Row row, Column column, Object value) throws InvalidValueException;

}
