/* Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.file.server;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import jp.littlesoft.file.common.SplitUploadParams;

/**
 * 普通のDiskFileを使用するFileItem
 * 
 * @author LittleSoft Corporation
 * @version 1.3.0
 */
@SuppressWarnings("serial")
public class UploadFileManager implements Serializable{
  
  private SplitUploadParams params;
  final private ArrayList<AbstractFileItem> items = new ArrayList<AbstractFileItem>();
  
  static class SplitFilesMergeException extends Exception{
    public SplitFilesMergeException(String msg) {
      super(msg);
    }
  };
  
  public UploadFileManager() {}
  
  /**
   * アップロードファイルのリストを取得する
   * @return
   */
  public List<AbstractFileItem> getItems() {
    return items;
  }


  
  /**
   * ファイル名を指定してアップロードファイルを取得する
   * @param fileName
   * @return
   */
  public AbstractFileItem getItemByFileName(String fileName) {
    for (AbstractFileItem item : items) {
      if (item.fileName.equals(fileName)) {
        return item;
      }
    }
    return null;
  }
  
  /**
   * フォーム項目名を指定してアップロードファイルを取得する
   * @param fieldName
   * @return
   */
  public AbstractFileItem getItemByFieldName(String fieldName) {
    for (AbstractFileItem item : items) {
      if (item.fieldName.equals(fieldName)) {
        return item;
      }
    }
    return null;
  }

  /**
   * アップロードパラメーターを取得します.<br/>
   * @return
   */
  public SplitUploadParams getParams() {
    return params;
  }
  
  /**
   * アップロードパラメーターを設定します.<br/>
   * 各ファイルアイテムにも設定されます
   * @param params
   */
  public void setParams(SplitUploadParams params) {
    this.params = params;
    for (AbstractFileItem item : items) {
      if (item.getParams()==null) {
        item.setParams(params);
      }
    }
  }
  
  /**
   * 指定されたインデクッスのアイテムを最終とする分割ファイルをマージする。
   * @param index
   */
  public void mergeSplitFiles(int index) throws Exception {
    AbstractFileItem[] itemArray = new AbstractFileItem[items.size()];
    items.toArray(itemArray);
    
    for (int i=index; i>0 ; i--) {
      // UploadParams is null はNG
      if (itemArray[i].getParams()==null) {
        throw new SplitFilesMergeException("Param is null:" + itemArray[i]);
      }
      if (!itemArray[i].getParams().isSplit) {
        throw new SplitFilesMergeException("File is not SplitUpload:" + itemArray[i]);
      }
      if (itemArray[i].getParams().splitSequence==0) {
        // 分割ファイルの先頭にきたらLoop終了
        break;
      } else {
        // 先頭でないアイテムはリストから削除
        items.remove(itemArray[i]);
      }
      // UploadParams is null はNG
      if (itemArray[i-1].getParams()==null) {
        throw new SplitFilesMergeException("Param is null:" + itemArray[i]);
      }
      // 分割ファイル名チェック
      if (!itemArray[i-1].fileName.equals(itemArray[i].fileName)) {
        throw new SplitFilesMergeException("Invalid file name:" + itemArray[i-1] + ";"+ itemArray[i]);
      }
      // 分割ファイルのシーケンスチェック
      if (itemArray[i-1].getParams().splitSequence != (itemArray[i].getParams().splitSequence-1)) {
        throw new SplitFilesMergeException("Invalid SplitSequence:" + itemArray[i-1] + ";"+ itemArray[i]);
      }
      itemArray[i].setPrevItem(itemArray[i-1]);
      itemArray[i-1].setNextItem(itemArray[i]);
    }
  }
  
}
