/* Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.soja;

import java.io.Serializable;
import java.lang.reflect.Array;
import java.math.BigDecimal;
import java.sql.DriverManager;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

import com.sun.org.apache.bcel.internal.generic.DLOAD;

import jp.littlesoft.TableTest.Employee;
import jp.littlesoft.data.EntityList;
import jp.littlesoft.data.ITableDef;
import jp.littlesoft.data.Column.Nullable;
import jp.littlesoft.data.ITableDef.MetaData;
import jp.littlesoft.data.ITableDef.TableDef;
import jp.littlesoft.sql.ISQLDataResolver;
import jp.littlesoft.sql.SQL;
import jp.littlesoft.sql.SchemaUpdator;

@SuppressWarnings("serial")
public class LSJsonTest {
  
  @TableDef(owner="SA", schema="PUBLIC")
  static public class UserGroup implements ITableDef {
    @ColumnDef(caption="グループID", nullable=Nullable.FALSE )
    public Integer id;
    @ColumnDef(caption="グループ名", defaultValue="[設定してください]", length=120, nullable=Nullable.FALSE)
    public String name;
    
    @ColumnDef(precision=15, scale=2 , defaultValue="0.12")
    public BigDecimal number;

    @Override
    public MetaData<?> createMetaData() {
      return new MetaData<UserGroup>() {{
        addPrimaryKey($$.id);
      }};
    }
  }
  
  @TableDef(owner="SA", schema="PUBLIC")
  static public class User implements ITableDef {
    @ColumnDef(caption="ユーザーID", nullable=Nullable.FALSE )
    public Long id;
    @ColumnDef(caption="ユーザー名", length=80, nullable=Nullable.FALSE)
    public String name;
    @ColumnDef(oldName="GROUP_ID", caption="グループID", nullable=Nullable.TRUE )
    public Integer gid;
    
    @Override
    public MetaData<?> createMetaData() {
      return new MetaData<User>() {{
        addPrimaryKey($$.id);
        addForeginKey($$.gid)
          .references(UserGroup.class)
          .onDelete(FKOption.RESTRICT)
          .onUpdate(FKOption.SET_NULL)
          .usingIndex(true);
        addIndex($$.name);
      }};
    }
  }
  
  @TableDef(schema="PUBLIC")
  static public class Base implements ITableDef {
    @ColumnDef(caption="登録日時" , formatPattern="yyyy/MM/dd hh:mm:ss.SSS")
    public java.sql.Timestamp createDatetime  = new java.sql.Timestamp(System.currentTimeMillis());
    @ColumnDef(caption="更新日時" , formatPattern="yyyy/MM/dd hh:mm:ss.SSS")
    public java.sql.Timestamp updateDatetime =  new java.sql.Timestamp(System.currentTimeMillis());
    @SuppressWarnings("unchecked")
    @Override
    public MetaData<?> createMetaData() {
      return new MetaData(this.getClass());
    }
  }
  
  static public class Pref extends Base {

    @ColumnDef(nullable=Nullable.FALSE)
    public Integer code;
    @ColumnDef(length=120, nullable=Nullable.FALSE)
    public String name;
    
    @Override
    public MetaData<?> createMetaData() {
      return new MetaData<Pref>() {{
        addPrimaryKey($$.code);
      }};
    }
  }

  static public class City extends Base {

    @ColumnDef(nullable=Nullable.FALSE)
    public Integer prefCode;
    @ColumnDef(nullable=Nullable.FALSE)
    public Short cityCode;
    @ColumnDef(length=100, nullable=Nullable.FALSE)
    public String name;
    
    @Override
    public MetaData<?> createMetaData() {
      return new MetaData<City>() {{
        addPrimaryKey($$.prefCode, $$.cityCode);
        addForeginKey($$.prefCode).references(Pref.class)
          .onUpdate(FKOption.SET_NULL)
          .onDelete(FKOption.CASCADE)
          .usingIndex(false);
      }};
    }
  }

  static public class Address extends Base {

    @ColumnDef(nullable=Nullable.FALSE)
    public Short prefCode;
    @ColumnDef(nullable=Nullable.FALSE)
    public Short cityCode;
    @ColumnDef(nullable=Nullable.FALSE)
    public Short addressCode;
    @ColumnDef(length=120, nullable=Nullable.FALSE)
    public String newStreetName;
    @ColumnDef(length=80, nullable=Nullable.TRUE)
    public String buildingName;
    @ColumnDef(length=20)
    public String tel;
    
    @Override
    public MetaData<?> createMetaData() {
      return new MetaData<Address>() {{
        addPrimaryKey($$.prefCode, $$.cityCode, $$.addressCode);
        addForeginKey($$.prefCode, $$.cityCode).references(City.class)
          .onUpdate(FKOption.RESTRICT)
          .onDelete(FKOption.CASCADE)
          .usingIndex(false);
        addIndex($$.newStreetName).unique(false);
        addIndex($$.buildingName).unique(false);
      }};
    }
  }
  

  
  public static class EmployeeInfo extends Employee {
    public static class Skill implements Serializable {
      public Skill(){}
      public Skill(java.sql.Date date,  String text) {
        this.date = date;
        this.text = text;
      }
      public java.sql.Date date;
      public String text;
    }
    
    public String[][] mailAddress;
    public Address address;
    public ArrayList<Skill> skills;
    public HashMap<String, String> famiily;
    @Override
    public MetaData<?> createMetaData() {
      return new MetaData<EmployeeInfo>(){{
        addPrimaryKey($$.empId);
      }};
    }
  }

  public static class Request  implements Serializable {
    public String sql;
    public List<?> result;
  }
  /**
   * @param args
   */
  @SuppressWarnings("unchecked")
  public static void main(String[] args) {
    try {
      Class.forName("org.h2.Driver");
      java.sql.Connection conn = DriverManager.getConnection("jdbc:h2:file:C:/gomi/mydb2", "sa", "");
      System.out.println("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@");
      
      new SchemaUpdator(conn).createOrReplaceTable(EmployeeInfo.class);
      
      SQL<EmployeeInfo> ins = new SQL<EmployeeInfo>() {{
        Employee e = $useDummy(Employee.class);
        //住所(Class)
        Address address = new Address();
        address.tel = "03-5372-1966";
        address.prefCode = 10;
        address.cityCode = 20;
        address.addressCode = 30;
        address.newStreetName = "豊島区池袋１－８－７";
        address.buildingName = "サン池袋I501";
        //メールアドレス(Array)
        String[][] mailAddress = new String[][] {
          {"abc@docomo.ne.jp","inq@littlesoft.jp"},
          {"xxx@docomo.ne.jp","AAA@littlesoft.jp"}
        };
        //学歴（List）
        ArrayList<EmployeeInfo.Skill> skills =  new ArrayList<EmployeeInfo.Skill>();
        skills.add(new EmployeeInfo.Skill(java.sql.Date.valueOf("1976-03-01"), "練馬区立泉新小学校"));
        skills.add(new EmployeeInfo.Skill(java.sql.Date.valueOf("1979-03-01"),"練馬区立三原台中学校"));
        skills.add(new EmployeeInfo.Skill(java.sql.Date.valueOf("1983-03-01"),"東京都立大泉北高等学校"));
        skills.add(new EmployeeInfo.Skill(java.sql.Date.valueOf("1987-03-01"),"早稲田大学社会科学部"));
        //家族(Map)
        HashMap<String, String> famiily = new HashMap<String, String>();
        famiily.put("長女", "ことね");
        famiily.put("長男", "まさ");
        famiily.put("妻", "まりなん");
        
        insert()
          .select(
            $(e.empId).into($$.empId),
            $(e.name).into($$.name),
            $(e.deptCode).into($$.deptCode),
            $(e.enterDate).into($$.enterDate),
            $(e.createDatetime).into($$.createDatetime),
            $(e.memo).into($$.memo),
            $(e.salary).into($$.salary),
            $(e.sex).into($$.sex),
            $(e.updateDatetime).into($$.updateDatetime),
            $(address).into($$.address),
            $(mailAddress).into($$.mailAddress),
            $(skills).into($$.skills),
            $(famiily).into($$.famiily)
          )
          .from(e)
        .prepare();
      }};
      ins.execute(conn, true, false);
      conn.commit();
      SQL<EmployeeInfo> select1 = new SQL<EmployeeInfo>() {{
        select();
      }};
      List<EmployeeInfo> empList = select1.execute(conn);
//      for (EmployeeInfo e : empList) {
//        e.address = new Address();
//        e.address.tel = "03-5372-1966";
//        e.address.prefCode = 10;
//        e.address.cityCode = 20;
//        e.address.addressCode = 30;
//        e.address.newStreetName = "豊島区池袋１－８－７";
//        e.address.buildingName = "サン池袋I501";
//      }
      Request req = new Request();
      req.sql = select1.toString();
      req.result = (EntityList<?>) empList;
      String result = LSJson.serialize(req, true);
      System.out.println(result);
      //List<EmployeeInfo> list = (List<EmployeeInfo>)LSJson.deserialize(result);
      
      Request list = (Request)LSJson.deserialize(result);
      System.out.println("-----------------------------------------------------------");
      System.out.println(LSJson.serialize(list, true));
      System.out.println("@@@ 単純SELECT complete");

    } catch (Exception e) {
      e.printStackTrace();
    }
    System.out.println("@@@ exit @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@");
    System.exit(0);
  }

}
