/* Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.tools;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.StringWriter;
import java.io.Writer;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Properties;

import jp.littlesoft.data.Column;
import jp.littlesoft.sql.SQL;
import jp.littlesoft.sql.SQLAdaptor;

public class SQLEntityClassWriter {
  
  private SQLAdaptor sqla = new SQLAdaptor();
  
  public SQLEntityClassWriter(Connection conn) throws Exception {
    sqla.setConnection(conn);
  }
  
  
  public SQLEntityClassWriter(String driver, String url, String user, String password, Properties p) throws Exception {
    sqla.connect(driver, url, user, password, p);
  }

  static String I2 = "  ";
  static String I4 = I2 + I2;
  static String I6 = I4 + I2;
  static String I8 = I6 + I2;
  public void execute(Writer out, String schema) throws Exception {
    try {
      StringWriter sw = new StringWriter();
      HashSet<String> imports = new HashSet<String>();
      imports.add("java.io.Serializable");

      Connection conn = sqla.getConnection();
      DatabaseMetaData meta = conn.getMetaData();
      ResultSet scheams = meta.getSchemas();
      while (scheams.next()) {
        if (schema!=null && !scheams.getString(1).equals(schema)) {
          continue;
        }
        sw.write("public class DB implements Serializable"); 
        //sw.write(SQL.underscoreToCamel(scheams.getString(2) + "_" + scheams.getString(1) , true));
        sw.write(" {\n");

        ResultSet tables = meta.getTables(scheams.getString(2), scheams.getString(1), null, null);
        while (tables.next()) {
          sw.write(I2);
          sw.write("@TableDef(name=\"");
          sw.write(tables.getString(3));
          sw.write("\")\n");
          sw.write(I2);
          sw.write("static public class ");
          sw.write(SQL.underscoreToCamel(tables.getString(3), true));
          sw.write(" implements Serializable {\n");
          ResultSet columns = meta.getColumns(tables.getString(1), tables.getString(2), tables.getString(3), null);
          while (columns.next()) {
            Class<?> clazz = Column.colTypeToClassMap(Column.mapToJava(columns.getInt(5)));
            imports.add(clazz.getName());
            sw.write(I4);
            sw.write("@ColumnDef(name=\"");
            sw.write(columns.getString(4));
            sw.write("\")\n");
            
            sw.write(I4);
            sw.write("public ");
            sw.write(clazz.getSimpleName() + " " + SQL.underscoreToCamel(columns.getString(4), false));
            sw.write(";\n");
          }
          sw.write(I2);
          sw.write("}\n");
        }
        sw.write("}\n");
      }
      ArrayList<String> l = new ArrayList<String>(imports);
      java.util.Collections.sort(l);
      for (String im : l) {
        out.write("import ");
        out.write(im);
        out.write(";\n");
      }
      out.write("import jp.littlesoft.sql.SQL.ColumnDef;\n");
      out.write("import jp.littlesoft.sql.SQL.TableDef;\n");
      out.write("\n");
      out.write(sw.toString());
      
    } finally {
      sqla.close();
    }
  }
  

  /**
   * @param args
   */
  public static void main(String[] args) {
    String driver = "";
    String url = "";
    String user = "";
    String password = "";
    String file = "";
    String schema = null;
    
    for (String arg : args) {
      if (arg.startsWith("d=")) driver  = arg.substring("d=".length());
      if (arg.startsWith("u=")) url     = arg.substring("u=".length());
      if (arg.startsWith("i=")) user    = arg.substring("i=".length());
      if (arg.startsWith("p=")) password = arg.substring("p=".length());
      if (arg.startsWith("f=")) file    = arg.substring("f=".length());
      if (arg.startsWith("s=")) schema  = arg.substring("s=".length());
    }
    try {
      SQLEntityClassWriter entityWriter = new SQLEntityClassWriter(driver, url, user, password, null);
      Writer writer = null;
      if ("".equals(file)) {
        writer = new BufferedWriter(new java.io.OutputStreamWriter(System.out));
      } else {
        File f = new java.io.File(file);
        FileOutputStream fos = new java.io.FileOutputStream(f);
        OutputStreamWriter osw = new java.io.OutputStreamWriter(fos, "UTF-8");
        writer = new java.io.BufferedWriter(osw);
      }
      entityWriter.execute(writer, schema);
      writer.flush();
      writer.close();
    } catch (Exception x) {
      x.printStackTrace();
    } finally {
      
    }
    System.exit(0);
  }

}
