/* Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.web;

import java.io.Serializable;
import java.util.ArrayList;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;

import jp.littlesoft.file.common.Attributes;
import jp.littlesoft.soja.LSJson;

@SuppressWarnings("serial")
public class DefaultJNLP implements Serializable {
  
//  <?xml version="1.0" encoding="UTF-8"?>
//  <jnlp spec="1.0+" codebase="${codebase}">
//  <information>
//      <title>${title}</title>
//    <vendor>${vendor}</vendor>
//    <offline-allowed/>
//  </information>
//  <security>
//    <all-permissions />
//  </security>
//  <resources>
//    <j2se version="${j2seVer}"/>
//    ${jars}
//  </resources>
//  <application-desc main-class="${mainClass}">
//    <argument>${applicationParams}</argument>
//    </argument>
//    </application-desc>
//  </jnlp>

  public String mainClass;
  public String codebase = null;
  public String title = "Java Web Start Apllication";
  public String vendor = "LittleSoft Corporation";
  public String j2seVer = "1.6+";
  public String templateName = "common/default.jnlp";
  public String jnlpFileName = null;
  final public ArrayList<String> jars = new ArrayList<String>() {
    @Override
    public String toString() {
      String result = "";
      for (String jar : jars) {
        result += "    <jar href=\"" + jar + "\"/>\n";
      }
      return result;
    }
  };
  final public Attributes appParams = new Attributes();
  private VelocityContext velocityContext = new VelocityContext();
  private HttpServletRequest request;
  private HttpServletResponse response;
  
  public DefaultJNLP(ActionPage actionPage) {
    this.request = actionPage.request;
    this.response = actionPage.response;
    this.response.setCharacterEncoding("UTF-8");
    this.response.setContentType("application/x-java-jnlp-file");
    codebase = toDefaultCodeBase(this.request);
    appParams.webappUrl = toURL(this.request);;
    appParams.jsessionId = actionPage.getJsessionID();
    appParams.sequence = actionPage.getSequenceValue(false);
    appParams.subsessionId = (actionPage.subSession!=null) ? actionPage.subSession.getID() : "";
    appParams.actionPageId = actionPage.getActionPageId();
    appParams.callbackMethod = "onJavaAppCallBack";
  }

  public void write() {
    try {
      // JNLPファイル名のセット
      if (jnlpFileName != null) {
        this.response.setHeader("Content-Disposition", "filename=" + jnlpFileName);
      }
      velocityContext.put("codebase", codebase);
      velocityContext.put("title", title);
      velocityContext.put("vendor", vendor);
      velocityContext.put("j2seVer", j2seVer);
      velocityContext.put("jars", jars.toString());
      velocityContext.put("mainClass", mainClass);
      velocityContext.put("appParams", LSJson.serialize(appParams, true));
      Template template;
      //template = Velocity.getTemplate(templateName);
      VelocityEngine engine = (VelocityEngine)request.getAttribute(ActionController.RA_VELOCITY_ENGINE);
      template = engine.getTemplate(templateName);
      template.merge(velocityContext, response.getWriter());
    } catch (Exception x) {
      x.printStackTrace(System.err);
      throw new RuntimeException(x);
    } finally {
      try {
        response.getWriter().flush();
        response.getWriter().close();
      } catch (Exception x) {}
    }

  }
  
  static public String toDefaultCodeBase(HttpServletRequest request) {
    String url = request.getScheme() + "://" + request.getServerName();
    if (request.getServerPort()!=80 && request.getServerPort()!=443) {
      url += ":" + request.getServerPort();
    }
    url += "/jws";
    return url;
  }

  static public String toURL(HttpServletRequest request) {
    String url = request.getScheme() + "://" + request.getServerName();
    if (request.getServerPort()!=80 && request.getServerPort()!=443) {
      url += ":" + request.getServerPort();
    }
    url += request.getRequestURI() + request.getContextPath();
    return url;
  }
}
