/* Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.web;

import java.io.Serializable;

@SuppressWarnings("serial")
public interface IDialogClient {
  
  static public class Error extends Exception {
    public Error(String message) {
      super(message);
    }
  }
  
  static public class Confirm extends Exception {
    public Confirm(String message) {
      super(message);
    }
  }
  
  //static public String 
  /**
   * ダイアログに引き渡すデータを決定します.<br/>
   * @param dialog 当メソッドをコールしたダイアログウィンドウ
   */
  public Serializable getDialogClientData(DefaultDialog<?> dialog);
  
  /**
   * DefaultDialogを継承したダイアログから返されるデータ(data)の内容をチェックします.<br/>
   * 当インターフェースを実装するアプリケーションクラスでデータチェック等の処理を実装してください.<br/>
   * エラーが検出した際は、適切なメッセージとともにIDialogClient.Errorをスローしてください.
   * これによりディフォルトのエラーメッセージボックスがブラウザに表示されます.<br/>
   * また、ユーザーの確認（YES/NO）を必要とすろ場合はIDialogClient.Confirmをスローしてください.
   * これによりYES/NOの確認メッセージボックスがブラウザに表示されます.<br/>
   * 内容に問題無い場合は戻り値にクライアント(ブラウザ)で実行されるJavaScriptのコードを返します.
   * 
   * @param dialog 当メソッドをコールしたダイアログウィンドウ
   * @param button ダイアログウィンドウで選択されたボタン
   * @param data   ユーザー入力内容を格納したデータオブジェクト
   * @throws DialogClient.Error, IDialogClient.Confirm;
   * @return JavaScript クライアント(ブラウザ)で実行されるJavaScript
   */
  public String evaluateDialogResult(DefaultDialog<?> dialog, DefaultDialog.Button button, Serializable data) 
     throws IDialogClient.Error, IDialogClient.Confirm;

}
