/* Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.web;

import java.io.IOException;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class ResourceFilter implements Filter {

  private static String startTime;
  private String root = "webres";
  private static int BUFF_SIZE = 4096;
  static {
    SimpleDateFormat format = new SimpleDateFormat("EEE, d MMM yyyy HH:mm:ss z", new Locale("JP", ""));
    synchronized (format) {
      format.setTimeZone(TimeZone.getTimeZone("GMT"));
      startTime = format.format(new Date());
    }
  }

  @Override
  public void destroy() {
  }

  @Override
  public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) throws IOException, ServletException {
    HttpServletRequest req = (HttpServletRequest) request;
    String mimeType = getMimeType(req);
    if (mimeType == null) {
      chain.doFilter(request, response);
      return;
    }
    response.setContentType(mimeType);
    HttpServletResponse res = (HttpServletResponse) response;
    res.setHeader("Cache-Control", "max-age=10");
    res.setHeader("Last-Modified", startTime);
    String path = req.getRequestURI();
    path = root + (path.startsWith("/") ? path : "/" + req.getRequestURI());
    InputStream in = this.getClass().getClassLoader().getResourceAsStream(path);
    if (in == null) {
      throw new ServletException("Resource not found [" + path + "]");
    }
    ServletOutputStream out = response.getOutputStream();
    try {
      byte[] buffer = new byte[BUFF_SIZE];
      int length = 0;
      while ((length = in.read(buffer)) >= 0) {
        out.write(buffer, 0, length);
      }
    } finally {
      out.flush();
      out.close();
    }
  }

  @Override
  public void init(FilterConfig config) throws ServletException {
    String pname = this.getClass().getName() + ".resRoot";
    if (config.getInitParameter(pname)!=null) {
      root = config.getInitParameter(pname);
      root = root.startsWith("/") ? root.substring(1) : root;
    }
  }

  public static String getMimeType(HttpServletRequest request) {
    String file = request.getRequestURI();
    file = file.substring(1);
    int index = file.lastIndexOf('.');
    String suffix;
    if (index >= 0) {
      suffix = file.substring(index + 1);
    } else {
      suffix = "";
    }
    suffix = suffix.toLowerCase();
    String mimeType;
    if ("ico".equals(suffix))
      mimeType = "image/x-icon";
    else if ("gif".equals(suffix))
      mimeType = "image/gif";
    else if ("jpg".equals(suffix))
      mimeType = "image/jpeg";
    else if ("jpe".equals(suffix))
      mimeType = "image/jpeg";
    else if ("jpeg".equals(suffix))
      mimeType = "image/jpeg";
    else if ("png".equals(suffix))
      mimeType = "image/png";
    else if ("bmp".equals(suffix))
      mimeType = "image/bmp";
    else if ("pdf".equals(suffix))
      mimeType = "application/pdf";
    else if ("doc".equals(suffix))
      mimeType = "application/msword";
    else if ("xls".equals(suffix))
      mimeType = "application/vnd.ms-excel";
    else if ("ppt".equals(suffix))
      mimeType = "application/vnd.ms-powerpoint";
    else if ("css".equals(suffix))
      mimeType = "text/css";
    else if ("js".equals(suffix))
      mimeType = "text/javascript";
    else if ("html".equals(suffix) || "htm".equals(suffix))
      mimeType = "text/html";
    else
      mimeType = null;
    return mimeType;
  }
}
