/* Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.web;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.sql.Connection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import jp.littlesoft.data.Column;
import jp.littlesoft.data.ITableDef;
import jp.littlesoft.data.Table;
import jp.littlesoft.data.event.TableEvent;
import jp.littlesoft.data.event.TableEvent.Operation;
import jp.littlesoft.sql.ISQLApplication;
import jp.littlesoft.sql.SQL;
import jp.littlesoft.sql.SQL.BindParameter;
import jp.littlesoft.sql.SQL.Comparer;
import jp.littlesoft.sql.SQL.Condition;
import jp.littlesoft.sql.SQL.Delete;
import jp.littlesoft.sql.SQL.Expression;
import jp.littlesoft.sql.SQL.Insert;
import jp.littlesoft.sql.SQL.Select;
import jp.littlesoft.sql.SQL.Update;

/**
 * SQL対応 アクションページの基底クラス。<br/>
 * 
 * 当クラスを継承して機能毎のアクションページを作成します。<br/>
 * このクラスはデータベースへのアクセスを容易にする為、SQLに関連する様々なメソッドを提供します。
 * 
 * @author LittleSoft Corporation
 * @version 1.3.0
 */
@SuppressWarnings({ "serial", "unchecked" })
public class SQLActionPage extends ActionPage implements ISQLApplication {
  
  private SQL sqlContext = new SQL();

  public SQLActionPage() {}

  /**
   * コンストラクタ<br/>
   * アプリケーションプログラムは必ずこのコンストラクタを使用します。
   * @param request
   * @param response
   */
  public SQLActionPage(HttpServletRequest request, HttpServletResponse response) {
    super(request, response);
  }

  /**
   * SQL式を開始します.<br/>
   * @param source ダミーエンティティオブジェクトのフィールドまたはgetter、あるいは式（Expression）オブジェクト
   */
  @Override
  public Expression $(Serializable source) {
    return sqlContext.$(source);
  }

  /**
   * 条件式を開始します.<br/>
   * @param nestedCondition 入れ子にする条件式（Conditionオブジェクト）
   */
  @Override
  public Condition<?> $if(Condition nestedCondition) {
    return sqlContext.$if(nestedCondition);
  }

  /**
   * 条件式を開始します.<br/>
   * @param value  ダミーエンティティオブジェクトのフィールドまたはgetter
   */
  @Override
  public Comparer<?,?> $if(Serializable value) {
    return sqlContext.$if(value);
  }

  /**
   * 実行時パラメータがNOT NULLであれば評価される条件式を開始します.<br/>
   * @param nestedCondition 入れ子にする条件式（Conditionオブジェクト）
   */
  @Override
  public Condition $evaluateNonNulls(Condition<?> nestedCondition) {
    return sqlContext.$evaluateNonNulls(nestedCondition);
  }  
  
  /**
   * 実行時パラメータがNOT NULLであれば評価される条件式を開始します.<br/>
   * @param value  ダミーエンティティオブジェクトのフィールドまたはgetter
   */
  @Override
  public Comparer $evaluateNonNulls(Serializable value) {
    return sqlContext.$evaluateNonNulls(value);
  }
  
  /**
   * バインドパラメータを作成します.<br/>
   * @param パラメータ名
   */
  @Override
  public BindParameter $bindParam(String name) {
    return sqlContext.$bindParam(name);
  }

  /**
   * ダミーエンティティオブジェクトを作成します。.<br/>
   * @param type エンティティオブジェクトのクラス
   */
  @Override
  public <E extends Serializable> E $useDummy(Class<? extends E> type) {
    return (E) sqlContext.$useDummy(type);
  }

  
  /**
   * ダミーエンティティオブジェクトにリモートデータベースのIDを割り当てます.<br/>
   * @param dummy  ダミーエンティティオブジェクﾄ
   * @param id  リモートデータベースを識別する任意のID
   */
  @Override
  public void $useRemote(ITableDef dummy, String id) {
    sqlContext.$useRemote(dummy, id);
  }
  /**
   * SELECTオブジェクトを作成します。.<br/>
   * columnsには選択するデータ項目[SELECT ...]を指定します。<br/>
   * 省略すると[SELECT * FROM ...] と同じ意味になります。
   * @param columns ダミーエンティティオブジェクトのフィールドまたはgetter、あるいは式（Expression）オブジェクト
   */
  @Override
  public Select select(Serializable... columns) {
    return sqlContext.select(columns);
  }

  /**
   * ダミーエンティティオブジェクトのフィールドから対応するカラムオブジェクトを取得します.<br/>
   */
  @Override
  public Column $column(Serializable dummyEntityFieldValue) {
    try {
      for (Field f : this.getClass().getFields()) {
        Serializable fobj = (Serializable) f.get(this);
        if (fobj instanceof Table<?>) {
          Table<?> t = (Table<?>)fobj;
          for (Column c : t.getColumnArray()) {
            if (c.getEntityValue(t.$$)==dummyEntityFieldValue) {
              return c;
            }
          }
        }
      }
    } catch (Exception x) {
      throw new RuntimeException(x);
    }
    return null;
  }
  /**
   * DELETE文を構築します.<br/>
   * @param entity
   * @return Delete
   */
  public Delete delete(Serializable entity) {
    return sqlContext.delete(entity);
  }
  
  /**
   * INSERT文を構築します.<br/>
   * @param entity
   * @return Insert
   */
  public Insert insert(Serializable entity) {
    return sqlContext.insert(entity);
  }
  
  /**
   * UPDATEオブジェクトを作成します。.<br/>
   * @param entity 更新対象テーブルのダミーエンティティオブジェクト
   * @return Update オブジェクト
   */
  public Update update(Serializable entity) {
    return sqlContext.update(entity);
  }

  /**
   * 設定済のダイナミック条件式を全てクリアします。
   */
  @Override
  public void resetDynamicCondtions() {
    sqlContext.resetDynamicCondtions();
  }

  /**
   * キーを指定してダイナミック条件式を設定します.<br/>
   * @param key 条件式を識別するキー
   */
  @Override
  public void setDynamicCondtion(String key, Condition<?> condition) {
    sqlContext.setDynamicCondtion(key, condition);
  }

  @Override
  public <E extends Serializable> void dataChanged(TableEvent<?> e, Operation operation) {
    // TODO Auto-generated method stub
  }

  /**
   * リモートデータベースに接続するダミーエンティティオブジェクトを作成します。.<br/>
   * @param type エンティティオブジェクトのクラス
   * @param id   リモートデータベースを識別する任意のID
   */
  @Override
  public <E extends Serializable> E $useRemoteDummy(Class<? extends E> type, String id) {
    return (E) sqlContext.$useRemoteDummy(type, id);
  }

  /**
   * リモートデータベースのコネクションを設定します.
   * @param id リモートデータベースを識別する任意のID
   * @param connection JDBCコネクション
   */
  @Override
  public void setRemoteConnection(String id, Connection connection) {
    sqlContext.setRemoteConnection(id, connection);
  }
}
