/* Copyright (c) 2006-2010 LittleSoft Corporation
 * 
 * LittleSoft J-Programming
 * 当プログラムの使用許諾については以下を参照してください。
 * http://www.littlesoft.jp/lsj/license.html
 */
package jp.littlesoft.web;

import java.io.IOException;
import java.io.Serializable;
import java.io.StringWriter;

import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.exception.MethodInvocationException;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.runtime.resource.loader.StringResourceLoader;
import org.apache.velocity.runtime.resource.util.StringResourceRepository;

/**
 * Stringで定義するVelocityテンプレートクラス.<br/>
 * @author littlesoft
 *
 */
@SuppressWarnings("serial")
public class StringVelocityTemplate implements Serializable {
  private static VelocityEngine engine = new VelocityEngine(); 
  private static StringResourceRepository repository = StringResourceLoader.getRepository();
  static {
    engine.addProperty(VelocityEngine.RESOURCE_LOADER, "STRING");
    engine.addProperty("STRING.resource.loader.class",
    org.apache.velocity.runtime.resource.loader.StringResourceLoader.class.getName());
    try { engine.init(); } catch (Exception e) { e.printStackTrace(); }
  }
  private String id = this.hashCode() + ":" + System.currentTimeMillis();
  
  /**
   * コンストラクタ.<br/>
   */
  public StringVelocityTemplate() {
    
  }
  
  /**
   * コンストラクタ.<br/>
   * @param resource Velocityテンプレート
   */
  public StringVelocityTemplate(String resource) {
    setResource(resource);
  }
  
  /**
   * StringでVelocityテンプレートを設定します.<br/>
   * @param resource
   */
  public void setResource(String resource) {
    repository.putStringResource(id , resource);
  }
  
  /**
   * VelocityテンプレートをStringで取得します.<br/>
   * @return
   */
  public String getResource() {
    return repository.getStringResource(id).getBody();
  }
  
  @Override
  protected void finalize() {
    repository.removeStringResource(id);
  }
  
  /**
   * テンプレートとVelocityContextの値を合成します。<br/>
   * @param context
   * @return マージされたString
   * @throws Exception
   */
  public String getMergedString(VelocityContext context) throws Exception {
    Template template = engine.getTemplate(id, "UTF-8"); 
    StringWriter writer = new StringWriter(); 
    template.merge(context, writer);
    writer.close(); 
    return writer.toString();
  }
}
